<?php
    namespace Util;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use \Util\Exception;
use DateTime;

class Util extends \Zeedhi\Framework\Controller\Simple {
    protected $entityManager;
    protected $instanceManager;
    protected $sessionService;
    protected $habilitaLogVenda;
    public $utilAPI;
    protected $databaseUtil;
    protected $tipoBanco;

    public function __construct(
        \Doctrine\ORM\EntityManager $entityManager,
        \Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager,
        \Helpers\Environment $sessionService,
        \Odhen\API\Util\Util $utilAPI,
        \Odhen\API\Util\Database $dbUtil
        ){
        $this->entityManager = $entityManager;
        $this->instanceManager = $instanceManager;
        $this->sessionService = $sessionService;
        $this->utilAPI = $utilAPI;
        $this->databaseUtil = $dbUtil;
        $this->tipoBanco = $this->instanceManager->getParameter('connection_params')['driver'];
        $this->habilitaLogVenda = $this->instanceManager->getParameter('HABILITA_LOG_VENDA');
    }

    const ORACLE = 'oci8';

    public function getParams ($params) {

        $treatedParams = array();
        foreach ($params as $param) {

            if (!empty($param['value'])) {
                if ($param['value'] === '=') {
                    $param['value'] = '';
                }
            }
            $treatedParams[$param['columnName']] = $param['value'];
        }

        return $treatedParams;
    }

    public function checkVersion($frontVersion){
        try {
            $versionOk = false;
            $updateVersion = false;

            // Armazena o arquivo onde contém a versão do backend.
            $versionFile = json_decode(file_get_contents(str_replace('\\', DIRECTORY_SEPARATOR, __DIR__. '/../../version.json')), true);

            // Identifica a versão informada no backend.
            $backVersion = $versionFile['backVersion'];

            // Compara se a versão esta igual a do frontend, caso esteja o sistem segue com o processo de login.
            if (version_compare($backVersion, $frontVersion, "==")) $versionOk = true;

            // Caso a versão do aplicativo esteja desatualizada em relação ao backend, o sistema informa ao usuário para que seja atualizado.
            // Pela regra, sera necessario o cliente atualizar o aplicativo, pois não será possível prosseguir caso não tenha atualizado.
            if (version_compare($backVersion, $frontVersion, ">")){
                throw new \Exception("Seu aplicativo está desatualizado. Atualize para a versão " . $backVersion . ".", 1);
            }

            // Caso o backend esteja desatualizado, o utilizador é solicitado para atualizá-lo.
            if (version_compare($backVersion, $frontVersion, "<")){
                if (!$this->instanceManager->getParameter('SELF_UPDATE')){
                    throw new \Exception("O servidor do aplicativo encontra-se desatualizado. Entre em contato com o suporte da Teknisa para atualizar a versão.", 1);
                }
                $updateVersion = true;
            }

            return array(
                'versionOk' => $versionOk,
                'updateVersion' => $updateVersion
            );

        } catch(\Exception $e){
            return array(
                'versionOk' => false,
                'updateVersion' => false,
                'message' => $e->getMessage()
            );
        }
    }

    public function updateVersion(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $params = $this->getParams($params);
            $frontVersion = $params['frontVersion'];

            set_time_limit(0); // Remove o timeout da requisição.

            // Definição dos diretórios necessários no diretório temporario do sistema operacional.
            // O diretório temporário do sitema operacional foi definido para que minimize a possibilidade de dar problemas, principalemente caso algum diretório esteja aberto no Windows, por exemplo.
            $odhenTempPath = sys_get_temp_dir().DIRECTORY_SEPARATOR."odhen";
            $versionTempPath = sys_get_temp_dir().DIRECTORY_SEPARATOR."odhen".DIRECTORY_SEPARATOR."versoes";

            // Cria os diretórios, caso não existam.
            if (!file_exists($odhenTempPath)) mkdir($odhenTempPath, 0777, true);
            if (!file_exists($versionTempPath)) mkdir($versionTempPath, 0777, true);

            // Inicia o nome do arquivo para download.
            $zipFile = $versionTempPath.DIRECTORY_SEPARATOR.$frontVersion.'.zip';

            //@todo - deixar a versão do projeto dinamico $frontVersion

            // app.odhen.com é um servidor temporario onde está armazenado as versões do projeto para download.
            $f = file_put_contents($zipFile, fopen("http://docsodhen.teknisa.com/downloads/versoes/pos/" . $frontVersion . ".zip", 'r'), LOCK_EX);

            // Valida o download do projeto.
            if ($f === false) throw new \Exception("Não foi possível atualizar a versão. Verifique a sua conexão com a internet.", 1);

            // Inicia o projeto de extração do download.
            $zip = new \ZipArchive;
            $res = $zip->open($zipFile);

            if ($res === true){
                // Extrai o download.
                $zip->extractTo($versionTempPath.DIRECTORY_SEPARATOR.$frontVersion);
                // Inicia o processo de troca da versão do produto.
                $rootPath = str_replace('\\', DIRECTORY_SEPARATOR, __DIR__. '/../../../../../');

                $time = time();

                // Faz um backup do produto atual para o diretório temporario do sistema operacional.
                if (@rename($rootPath."odhenPOS", $versionTempPath.DIRECTORY_SEPARATOR.$backVersion.'_'.$time) == false){
                    throw new \Exception("Não foi possível atualizar o projeto: Verifique no servidor se há algum diretório ou arquivo aberto relacionado ao produto Odhen POS. (1)", 1);
                }

                // Configura o environment.xml do projeto novo usando os dados do antigo.
                if (@rename($versionTempPath.DIRECTORY_SEPARATOR.$backVersion.'_'.$time.DIRECTORY_SEPARATOR.'backend_75000'.DIRECTORY_SEPARATOR.'environment.xml', $versionTempPath.DIRECTORY_SEPARATOR.$frontVersion.DIRECTORY_SEPARATOR.'odhenPOS'.DIRECTORY_SEPARATOR.'backend_75000'.DIRECTORY_SEPARATOR.'environment.xml') == false){
                    @rename($versionTempPath.DIRECTORY_SEPARATOR.$backVersion.'_'.$time, $rootPath."odhenPOS");
                    throw new \Exception("Não foi possível atualizar o projeto: Não foi possível modificar os arquivos de configuração de banco. (2)", 1);
                }

                // Atualiza a versão do produto com a versão atualizada.
                if (@rename($versionTempPath.DIRECTORY_SEPARATOR.$frontVersion.DIRECTORY_SEPARATOR.'odhenPOS', $rootPath."odhenPOS") == false){
                    @rename($versionTempPath.DIRECTORY_SEPARATOR.$frontVersion.DIRECTORY_SEPARATOR.'odhenPOS'.DIRECTORY_SEPARATOR.'backend_75000'.DIRECTORY_SEPARATOR.'environment.xml', $versionTempPath.DIRECTORY_SEPARATOR.$backVersion.'_'.$time.DIRECTORY_SEPARATOR.'backend_75000'.DIRECTORY_SEPARATOR.'environment.xml');
                    @rename($versionTempPath.DIRECTORY_SEPARATOR.$backVersion.'_'.$time, $rootPath."odhenPOS");
                    throw new \Exception("Não foi possível atualizar o projeto: Não foi possível mover o projeto atualizado para o diretório final. (3)", 1);
                }
            }
            else {
                // Verificar se o link de acesso ao projeto com a versão passada como argumento pelo front está funcionando.
                throw new \Exception("Não foi possível realizar o download do projeto. Entre em contato com o suporte da Teknisa para validar a versão.", 1);
            }

            // Após baixar e atualizar o produto, a requisição segue seu caminho.
            $response->addMessage(new Message("O servidor foi atualizado, faça o login novamente."));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('nothing', array(array('nothing' => 'nothing'))));
        } catch(\Exception $e){
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        }
    }

    public function intToHexa($number) {
        $number = intval($number);
        $number = dechex($number);
        $number = str_pad($number, 6, '0', STR_PAD_LEFT);
        $b = substr($number, 0, 2);
        $g = substr($number, 2, 2);
        $r = substr($number, 4, 2);
        $number = $r . $g . $b;
        return $number;
    }

    public function strToFloat($strValue) {
        $strValue = str_replace(",", ".", $strValue);
        return (float)$strValue;
    }

    public function testConnection(Request\Filter $request, Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();
            if (!empty($params)) $testDB = $params[0]['value'];
            else $testDB = false;

            if ($testDB){
                $filiais = $this->entityManager->getConnection()->fetchAssoc("SQL_TEST_FILIAIS", array());
                if (empty($filiais)){
                    $response->addMessage(new Message("Conexão efetuada, mas nenhum registro retornado."));
                }
                else {
                    $response->addMessage(new Message("Conexão efetuada. Registros retornados: " . $filiais['FILIAIS']));
                }
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('nothing', array(array('nothing' => 'nothing'))));
        } catch (\Exception $e) {
            Exception::logException($e);
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function saveRequests(Request\Filter $request, Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $times  = $params[0]['value'];
            $header = getallheaders();
            $times['userId'] = $header['User-Id'];
            self::log(json_encode($times));
        } catch (\Exception $e) {
            Exception::logException($e);
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function criaXMLCatraca($comanda, $evento, $caminho, $statusExterno){
    
$xml = <<<XML
<?xml version="1.0" standalone="yes" ?>
<DATAPACKET Version="2.0">
<METADATA>
<FIELDS>
<FIELD attrname="ID_COMANDA" fieldtype="string" WIDTH="20" />
<FIELD attrname="EVENTO" fieldtype="string" WIDTH="1" />
</FIELDS>
<PARAMS />
</METADATA>
<ROWDATA>
<ROW RowState="4" ID_COMANDA="{$comanda}" EVENTO="{$evento}" />
</ROWDATA>
</DATAPACKET>
XML;
        $dom = new \DOMDocument();
        $dom->loadXML($xml);
        file_put_contents($caminho . '/' . $comanda . $statusExterno . ".xml", $dom->saveXML());

    }

    public function getNewCode($CDCONTADOR, $size = 20){
        $params = array(
            ':CDCONTADOR' => $CDCONTADOR
        );
        $code = $this->entityManager->getConnection()->fetchAssoc("SQL_NOVO_CODIGO", $params);
        $code = substr($code['NRSEQUENCIAL'], strlen($code['NRSEQUENCIAL']) - $size);
        return $code;
    }

    public function newCode($counterName, $size = '20'){
        $nrseq = null;
        $connection = $this->entityManager->getConnection();
        $storedProcedure = new StoredProcedure($connection, 'NOVO_CODIGO');

        $storedProcedure->addParam(new Param('P_CONTADOR', Param::PARAM_INPUT, $counterName, Param::PARAM_TYPE_STR, strlen($counterName)));
        $storedProcedure->addParam(new Param('P_SEQUENCIAL', Param::PARAM_OUTPUT, $nrseq, Param::PARAM_TYPE_STR, 20));
        $storedProcedure->execute();
    }
    const FILLER = "0";

    public function zeroFill($var, $size) {
        return str_pad($var, $size, self::FILLER, STR_PAD_LEFT);
    }

    public function removeAcentos($texto) {
        $table = array('á' => 'a', 'à' => 'a', 'ã' => 'a', 'â' => 'a',
            'é' => 'e', 'ê' => 'e', 'í' => 'i', 'ó' => 'o', 'ô' => 'o',
            'õ' => 'o', 'ú' => 'u', 'ü' => 'u', 'ç' => 'c', 'Á' => 'A',
            'À' => 'A', 'Ã' => 'A', 'Â' => 'A', 'É' => 'E', 'Ê' => 'E',
            'Í' => 'I', 'Ó' => 'O', 'Ô' => 'O', 'Õ' => 'O', 'Ú' => 'U',
            'Ü' => 'U', 'Ç' => 'C_');
        return preg_replace("[^a-zA-Z0-9_]", "", strtr($texto, $table));
    }

    public function addSessionVar($value) {
        $this->sessionService->startSession($value);
    }

    public function setUserInfo($userInfo) {
        $this->sessionService->setUserInfo($userInfo);
    }

    public function getSessionVars($chave) {
        return $this->sessionService->getUserInfo($chave);
    }

    public function endSession() {
        return $this->sessionService->endSession();
    }

    public static function encrypt($text) {
        return md5('odhen'.$text.'7833');
    }

    public function log($info){
        $line = date("d-m-Y H:i:s") . " - " . $info . "\n";
        $date = date("dmY");
        file_put_contents(__DIR__ . '/../../../../../CDL/php_log_' . $date . ".txt", $line, FILE_APPEND);
    }

    public function replicate($char, $numero){
        $result = $char;
        for ($i = 1; $i < $numero; ++$i){
            $result .= $char;
        }
        return $result;
    }

    public function toRight($leftText, $rightText, $lineWidth){
        $spaceToFill = $lineWidth - (strlen($leftText) + strlen($rightText));
        $result = $leftText . self::replicate(' ', $spaceToFill);
        $result .= $rightText;
        return $result;
    }

    public function vazio($string){
        if (Empty($string)) {
            $string = ' ';
        }
        return $string;
    }

    public function truncate($val, $f="0") {
        if(($p = strpos($val, '.')) !== false) {
            $val = floatval(substr($val, 0, $p + 1 + $f));
        }
        return $val;
    }

    public function logFOS($filial, $caixa, $idOperFos, $cdOperador, $cdOperRespOp, $dsMotivoFos, $dsLivreFos){

        $params = array(
            $cdOperador
        );

        $answer = $this->entityManager->getConnection()->fetchAssoc("SQL_BUSCA_NOME_OPERADOR", $params);

        if ($cdOperRespOp != "") {

            $params = array(
                $cdOperRespOp
            );

            $response = $this->entityManager->getConnection()->fetchAssoc("SQL_BUSCA_NOME_OPERADOR", $params);

            $nmOperRespOp = $response['NMOPERADOR'];
        } else {
            $cdOperRespOp = $cdOperador;
            $nmOperRespOp = $answer['NMOPERADOR'];
        }

        if (empty($nmOperRespOp)) $nmOperRespOp = "";

        $params = array(
            $filial,
            $caixa,
            $idOperFos,
            $cdOperador,
            $answer['NMOPERADOR'],
            $cdOperRespOp,
            $nmOperRespOp,
            $dsMotivoFos,
            substr($dsLivreFos, 0, 250)
        );

        $this->entityManager->getConnection()->executeQuery("SQL_INSERE_LOG", $params);
    }

    public function formataPreco($valor){
        return number_format($this->truncate($valor, 2), 2, ",", "");
    }

    public function getNextUpdateTime($updateTimes){
        $date = date_create_from_format('H:i', date("H:i"));
        $currentTime = strtotime($date->format('Y-m-d H:i'));

        $nextUpdateTime = null;
        foreach ($updateTimes as $updateTime){
            if ($currentTime < $updateTime){
                $nextUpdateTime = $updateTime;
                break;
            }
        }

        return array(
            'nextUpdateTime' => !empty($nextUpdateTime) ? $nextUpdateTime : strtotime($date->format('Y-m-d 23:59')) + 60
        );
    }

    public function adjustDifference(&$array, $difference, $type, $propDesconto){
        $apportionmentAmount = intval($difference / 0.01);
        $i = 0;
        while ($apportionmentAmount > 0) {
            $apportionmentAmount = self::changeValue($array, $apportionmentAmount, $type, $propDesconto);
            $i++;
            if ($i == 1000) throw new \Exception("Erro no rateio do desconto/gorjeta.");
        }
    }

    private function changeValue(&$array, $apportionmentAmount, $type, $propDesconto){
        foreach ($array as &$arrayValue) {
            if ($type == 'prodObject'){
                $vrProd = self::calculaTotalItem($arrayValue);
                if ($vrProd > 0.01){
                    $arrayValue[$propDesconto] += 0.01;
                    $apportionmentAmount--;
                    if ($apportionmentAmount == 0) {
                        break;
                    }
                }
            } else {
                $arrayValue += 0.01;
                $apportionmentAmount--;
                if ($apportionmentAmount == 0) {
                    break;
                }
            }
        }
        return $apportionmentAmount;
    }

    public function calculaTotalItem($produtoAtual){
        if (isset($produtoAtual['VRDESITVEND'])){
            return round(floatval(bcmul(str_replace(',','.',strval($produtoAtual['VRUNITVEND'] + $produtoAtual['VRUNITVENDCL'])), str_replace(',','.',strval($produtoAtual['QTPRODVEND'])), '2')) + $produtoAtual['VRACRITVEND'] - $produtoAtual['VRDESITVEND'], 2);
        } else {
            return round(floatval(bcmul(str_replace(',','.',strval($produtoAtual['VRPRECCOMVEN'] + $produtoAtual['VRPRECCLCOMVEN'])), str_replace(',','.',strval($produtoAtual['QTPRODCOMVEN'])), '2')) + $produtoAtual['VRACRCOMVEN'] - $produtoAtual['VRDESCCOMVEN'], 2);
        }
    }

    // rotina criada para aplicar o desconto da promoção inteira (todo o desconto somado) em certos produtos da promoção
    // colocada no Util pois modo Mesa e Balcão compartilha desta rotina em diferentes pontos
    public function validaDescontoDiferenciado($CDFILIAL, $CDPRODPROMOCAO, &$composicao, $propDesconto){
        $descontoAplicado = false;
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDPRODPROMOCAO' => $CDPRODPROMOCAO
        );
        $prodPromocao = $this->entityManager->getConnection()->fetchAll("SQL_PROMOCAO_APLICADESCFIL", $params);
        if (empty($prodPromocao)){
            $prodPromocao = $this->entityManager->getConnection()->fetchAll("SQL_PROMOCAO_APLICADESC", $params);
        }

        // valida se a promoção controla desconto diferenciado
        if (!empty($prodPromocao)){
            $prodPromocao = array_unique(array_column($prodPromocao, 'CDPRODUTO'));

            // valida se algum produto pedido está para ter desconto diferenciado
            if (array_intersect(array_column($composicao, 'CDPRODUTO'), $prodPromocao)){
                $prodComDesconto = $prodSemDesconto = array();
                foreach ($composicao as $produto) {
                    if (in_array($produto['CDPRODUTO'], $prodPromocao)){
                        $prodComDesconto[] = $produto;
                    } else {
                        $prodSemDesconto[] = $produto;
                    }
                }
                // valida se o desconto já não está rateado
                if (!empty($prodSemDesconto)){
                    $descontoAplicado = $this->aplicaDescontoDiferenciado($prodComDesconto, $prodSemDesconto, $propDesconto);
                    if ($descontoAplicado){
                        // merge dos produtos modificados
                        $composicao = array_merge($prodComDesconto, $prodSemDesconto);
                    }
                }
            }
        }

        return $descontoAplicado;
    }

    private function aplicaDescontoDiferenciado(&$prodComDesconto, &$prodSemDesconto, $propDesconto){
        $valorDesconto = 0;
        foreach ($prodSemDesconto as $produto) {
            $valorDesconto += $produto[$propDesconto];
        }

        $vrProdComDesc = 0;
        foreach ($prodComDesconto as $produto) {
            $vrProdComDesc += round($this->calculaTotalItem($produto) - 0.01, 2);
        }

        // valida se existe desconto a ser rateado e se o valor do desconto é menor que o valor permitido para rateio
        if ($valorDesconto > 0 && $vrProdComDesc >= $valorDesconto){
            foreach ($prodSemDesconto as &$produto) {
                $produto[$propDesconto] = 0;
            }

            $porcVrDesc = $valorDesconto / $vrProdComDesc;
            $descontoAplicado = 0;

            // aplica rateio desconto nos produtos
            foreach ($prodComDesconto as &$produto) {
                $porcVrDescProd = floatval(bcmul(strval((1 / $produto['QTPRODCOMVEN'])), strval($porcVrDesc), 2));
                $descPorProd = floatval(bcmul(str_replace(',','.',strval($porcVrDescProd)), str_replace(',','.',strval($this->calculaTotalItem($produto))), '2'));

                $produto[$propDesconto] += $descPorProd;
                $descontoAplicado += $descPorProd;
            }
            if ($descontoAplicado < $valorDesconto){
                // ajusta casas decimais
                $this->adjustDifference($prodComDesconto, round($valorDesconto - $descontoAplicado, 2), 'prodObject', $propDesconto);
            }

            return true;
        } else {
            return false;
        }
    }

    public function databaseIsOracle() {
        return $this->tipoBanco == self::ORACLE;
    }

    public function LogBandeira($texto){
        if (!$this->utilAPI->isLinux()){
            $tipo = 'caminhoBandeira';
            $paths = ['LOG/', $tipo.'/'];
            $logPath = dirname(__DIR__) . '/../../../../';

            foreach ($paths as $path){
                $logPath = $logPath . $path;
                if (!is_dir($logPath)){
                    mkdir($logPath, 0700);
                }
            }

            $logPath = $logPath .'/'. (new \DateTime())->format('d-m-Y') . '.txt';
            $horaAtual = (new \DateTime())->format('d/m/Y H:i:s');
            file_put_contents($logPath, $horaAtual . ' - ' . $texto . "\r\n", FILE_APPEND);
        }
    }

    public function LogCRMBonus($texto){
        if (!$this->utilAPI->isLinux()){
            $tipo = 'respostaCRM';
            $paths = ['LOG/', $tipo.'/'];
            $logPath = dirname(__DIR__) . '/../../../../';

            foreach ($paths as $path){
                $logPath = $logPath . $path;
                if (!is_dir($logPath)){
                    mkdir($logPath, 0700);
                }
            }

            $logPath = $logPath .'/'. (new \DateTime())->format('d-m-Y') . '.txt';
            $horaAtual = (new \DateTime())->format('d/m/Y H:i:s');
            file_put_contents($logPath, $horaAtual . ' - ' . $texto . "\r\n", FILE_APPEND);
        }
    }

    public function logVendaTef(Request\Filter $request, Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $params = $this->getParams($params);
            $text = $params['text'];
            $CDCAIXA = isset($params['CDCAIXA'])?$params['CDCAIXA']:'';

            if($this->habilitaLogVenda) {
                $path = '';
                if($this->utilAPI->isLinux()){
                    $path = sys_get_temp_dir() . '/LOG' ;
                } else {
                    $path = __DIR__ . '/../../../../../LOG';
                }

                $fileName = 'VendaPOS - TEF - ' . date("d-m-Y");
                $content = date("d-m-Y H:i:s") . " - " . $CDCAIXA. ":" . $text . "\n\n";

                file_put_contents($path . '/' . $fileName . ".txt", $content, FILE_APPEND);
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('LogVendaTef', array()));
        } catch(\Exception $e){
            Exception::logException($e);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('LogVendaTef', array()));
        }
    }

    /*gera o log local, no diretorio LOG/($nomeDir)/ */
    public function geraLog($texto, $CDFILIAL, $CDCAIXA, $data, $tipoLog = null) {
        $this->utilAPI->geraLog($texto, $CDFILIAL, $CDCAIXA, $data, $tipoLog);
    }

    public function geraLog2($texto, $CDFILIAL=null, $CDCAIXA=null, $tipoLog=null){
		$this->utilAPI->geraLog2($texto, $CDFILIAL, $CDCAIXA, $tipoLog);
    }

    public static function deletarPasta($dir) {
        if(!file_exists($dir)){ return; }
        if(!is_dir($dir)){ unlink($dir); return; }
        $files = array_diff(scandir($dir), array('.','..'));
        foreach ($files as $file) {
            (is_dir("$dir/$file")) ? self::deletarPasta("$dir/$file") : unlink("$dir/$file");
        }
        return rmdir($dir);
    }

	public function copiarPasta($origem, $destino) {
		if (is_dir($origem)) {
			if (!is_dir($destino)) {
				mkdir($destino, 0777, true);
			}
			$conteudo = scandir($origem);
			foreach ($conteudo as $item) {
				if ($item != '.' && $item != '..') {
					self::copiarPasta($origem.DIRECTORY_SEPARATOR.$item, $destino.DIRECTORY_SEPARATOR.$item);
				}
			}
		} else {
			copy($origem, $destino);
		}
	}
	
	public function colunaExiste($TABELA, $COLUNA) {
		$params = array( 'TABELA' => $TABELA,'COLUNA' => $COLUNA );
		$colunaExiste = $this->entityManager->getConnection()->fetchAssoc('SQL_VERIFICA_CAMPO', $params);
		if($colunaExiste){
			if($colunaExiste['resultado']>0){
				return true;
			}
		}
		return false;
    }

    public function AtualizacaoDisponivel(Request\Filter $request, Response $response){
        $AtualizacaoDisponivel = null;
        try {
            // so busca atualizacao se o backend for diferente de saas e a base for diferente de oracle
            if(!$this->instanceManager->getParameter('ISSAAS') && $this->instanceManager->getParameter('connection_params')['driver'] != 'oci8'){
                $params = $request->getFilterCriteria()->getConditions();
                $params = $this->getParams($params);
                $frontVersion = $params['frontVersion'];

				$DSURLATUALIZACAO = "https://docsodhen.teknisa.com/downloads/versoes/pos/"; //"http://localhost:9090/downloads/versoes/pos/";

                //VERIFICA SE O CAMPO EXISTE SOMENTE PARA NÃO PRECISAR ESPERAR A HOMOLOgacao do campo no FosTab
                if(self::colunaExiste('PARAVEND','DSURLATUALIZACAO')){
					$arrUrlAtualizacao = $this->entityManager->getConnection()->fetchAssoc('SQL_GET_URL_ATUALIZACAO', $params);
					$DSURLATUALIZACAO = ($arrUrlAtualizacao)?$arrUrlAtualizacao['DSURLATUALIZACAO']:"";
					// se o DSURLATUALIZACAO esta como null, significa que a atualização automatica esta desativada para a quela filial. para ativar a atualização automatica novamente tem que parametrizar um valor no campo
                }
				
                if(!empty($DSURLATUALIZACAO)){
                    // o nome do arquivo no servidor deve ser "versao_odhenPOS" mas o caminho e URL pode ser customizado
					$DSURLATUALIZACAO = rtrim($DSURLATUALIZACAO, '/').'/versao_odhenPOS';
                    $curl = curl_init();
                    curl_setopt($curl, CURLOPT_URL, $DSURLATUALIZACAO);
                    curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'GET');
                    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
                    curl_setopt($curl, CURLOPT_ENCODING,'');
                    curl_setopt($curl, CURLOPT_MAXREDIRS,10);
                    curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
                    curl_setopt($curl, CURLOPT_HTTP_VERSION , CURL_HTTP_VERSION_1_1);
                    curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
                    curl_setopt($curl, CURLOPT_TIMEOUT, 14);
                    curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 14);
                    $curl_response = curl_exec($curl);
                    $err    = curl_errno($curl);
                    $errmsg = curl_error($curl);
                    $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
                    curl_close($curl);

                    if (($httpcode ==='404' || $httpcode === 404)||$err){
                        throw new \Exception("<br>Erro na requisição");
                    }

                    $arrVersaoServidor = explode('.',$curl_response);
                    $arrVersaoAtual = explode('.',$frontVersion);

                    $cont = (sizeof($arrVersaoAtual)>sizeof($arrVersaoServidor))?sizeof($arrVersaoAtual):sizeof($arrVersaoServidor);
					//verifica qual das duas versões retornadas é maior. se a maior versão for a do servidor, retorna ela se não, não retorna nada
                    for ($i=0; $i < $cont ; $i++) {
                        if(!isset($arrVersaoServidor[$i])){ $arrVersaoServidor[$i] = 0; }
                        if(!isset($arrVersaoAtual[$i])){ $arrVersaoAtual[$i] = 0;  }
                        if($arrVersaoServidor[$i]>$arrVersaoAtual[$i]){
                            $AtualizacaoDisponivel = implode('.',$arrVersaoServidor);
                            break;
                        }
                    }
                }
            }
        } catch (\Exception $e) {
            //var_dump($e->getMessage());
        }
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('AtualizacaoDisponivel', array(array('AtualizacaoDisponivel' => $AtualizacaoDisponivel))));
    }

    public function AtualizarVersao(Request\Filter $request, Response $response){
        try {
            set_time_limit(0); // Remove o timeout da requisição.

            $params = $request->getFilterCriteria()->getConditions();
            $params = $this->getParams($params);
            $novaVersao = $params['frontVersion'];

			$DSURLATUALIZACAO = "https://docsodhen.teknisa.com/downloads/versoes/pos/"; //"http://localhost:9090/downloads/versoes/pos/";

			//VERIFICA SE O CAMPO EXISTE SOMENTE PARA NÃO PRECISAR ESPERAR A HOMOLOgacao do campo no FosTab
			if(self::colunaExiste('PARAVEND','DSURLATUALIZACAO')){
				$arrUrlAtualizacao = $this->entityManager->getConnection()->fetchAssoc('SQL_GET_URL_ATUALIZACAO', $params);
				$DSURLATUALIZACAO = ($arrUrlAtualizacao)?$arrUrlAtualizacao['DSURLATUALIZACAO']:"";
				// se o DSURLATUALIZACAO esta como null, significa que a atualização automatica esta desativada para a quela filial. para ativar a atualização automatica novamente tem que parametrizar um valor no campo
			}

            if (empty($DSURLATUALIZACAO)) throw new \Exception("Não foi possível atualizar o projeto: Não foi parametrizada a URL de atualização.", 1);

            $rootPath = realpath(str_replace('\\', DIRECTORY_SEPARATOR, __DIR__. '/../../../../../')).DIRECTORY_SEPARATOR;

            $zipFile = $rootPath.'odhenPOS_'.$novaVersao.'.zip';
			if(file_exists($zipFile)){
				@unlink($zipFile);
			}

            //Baixa arquivo com padrao X.X.X.X.X.zip, caso não exista, tenta baixar com o padrao odhenPÒS_XXXXX.xip
            $f = @file_put_contents($zipFile, fopen($DSURLATUALIZACAO."/".$novaVersao.".zip", 'r'), LOCK_EX);
            if ($f === false || $f === 0){
                $f = @file_put_contents($zipFile, fopen($DSURLATUALIZACAO."/odhenPOS-v".str_replace(".", "", $novaVersao).".zip", 'r'), LOCK_EX);
            }

            if ($f === false || $f === 0) throw new \Exception("Não foi possível realizar o download da versão ".$novaVersao, 1);

			$novaVersao = str_replace(".", "", $novaVersao);

            $zip = new \ZipArchive;
            if ($zip->open($zipFile) === false) throw new \Exception("Arquivo zip do projeto inválido.", 1);
            self::deletarPasta($rootPath.'odhenPOS_'.$novaVersao.'_temp');
            self::deletarPasta($rootPath.'odhenPOS_'.$novaVersao);
            $zip->extractTo($rootPath.'odhenPOS_'.$novaVersao.'_temp');
			if (@rename($rootPath.'odhenPOS_'.$novaVersao.'_temp'.DIRECTORY_SEPARATOR.'odhenPOS', $rootPath.'odhenPOS_'.$novaVersao) == false) throw new \Exception("Não foi possível renomear o projeto da nova versão", 1);
            self::deletarPasta($rootPath.'odhenPOS_'.$novaVersao.'_temp');

            $listaBackendsAntigos = Array();
            $pastas = new \DirectoryIterator($rootPath.'odhenPOS');
            foreach($pastas as $pasta){
                if($pasta->gettype() === 'dir'){
                    $nome = $pasta->getFilename();
                    if(strpos($nome, 'backend_')!==false){
                        $listaBackendsAntigos[] = $pasta->getFilename();
                    }
                }
            }
            if(empty($listaBackendsAntigos[0])) throw new \Exception("Não foi possível localizar o backend da versão antiga", 1);

			rsort($listaBackendsAntigos);
			$versaoBackendAntigo = str_replace("backend_", "", $listaBackendsAntigos[0]);

            self::deletarPasta($rootPath.'odhenPOS_'.$versaoBackendAntigo);
            // Renomeia a versao atual
			if (@rename($rootPath.'odhenPOS', $rootPath.'odhenPOS_'.$versaoBackendAntigo) == false) throw new \Exception("Não foi possível renomear o projeto da versão atual.", 1);

			//Renomeia a nova versão para o nome padrão
			if (@rename($rootPath.'odhenPOS_'.$novaVersao, $rootPath.'odhenPOS') == false) throw new \Exception("Não foi possível renomear o projeto da nova versão.", 1);

			foreach($listaBackendsAntigos as $pasta){
				// Copia os backends antigos para nova versão para manter o cliente rodando caso tenha algum apk com versão muito antiga
				// as copias de backends antigos podem ser retiradas após a implemnentação do frontendapk
				try {
					self::copiarPasta($rootPath.'odhenPOS_'.$versaoBackendAntigo.DIRECTORY_SEPARATOR.$pasta,$rootPath.'odhenPOS'.DIRECTORY_SEPARATOR.$pasta);
				} catch(\Exception $e){}
			}

			//pega o environment da versão antiga e coloca em todos os backends da versão nova
			//após a implementação do frontendapk teremos somente o backend antigo e o novo. Não será nescessário fazer um loop em todos os backends
			$pastas = new \DirectoryIterator($rootPath.'odhenPOS');
			foreach($pastas as $pasta){
				if($pasta->gettype() === 'dir'){
					$nome = $pasta->getFilename();
					if(strpos($nome, 'backend_')!==false){
						if (@copy($rootPath.'odhenPOS_'.$versaoBackendAntigo.DIRECTORY_SEPARATOR.'backend_'.$versaoBackendAntigo.DIRECTORY_SEPARATOR.'environment.xml', $rootPath.'odhenPOS'.DIRECTORY_SEPARATOR.$nome.DIRECTORY_SEPARATOR.'environment.xml') == false){
							throw new \Exception("Não foi possível copiar o environment da versão antiga para a versão nova.", 1);
						}
					}
				}
			}
			$response->addMessage(new Message("O servidor foi atualizado, faça o login novamente."));
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('nothing', array(array('nothing' => 'nothing'))));
        } catch(\Exception $e){
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        }
    }

    public function getConfigJs(){
        $retorno = null;
        try {
            $rootPath = realpath(str_replace('\\', DIRECTORY_SEPARATOR, __DIR__. '/../../../../../')).DIRECTORY_SEPARATOR;
            if(file_exists($rootPath.'odhenPOS'.DIRECTORY_SEPARATOR.'mobile'.DIRECTORY_SEPARATOR.'config.js')){
                $configjs = file_get_contents($rootPath.'odhenPOS'.DIRECTORY_SEPARATOR.'mobile'.DIRECTORY_SEPARATOR.'config.js');
                if($configjs !== false && $configjs !== 0){
                    if(!empty($configjs)){
                        $configjs = substr($configjs,strpos($configjs,"{"));
                        $configjs = substr($configjs,0,strpos($configjs,"}")+1);

                        $configjs = str_replace("'",'"',$configjs);
                        $configjs = str_replace("\\n","",$configjs);
                        $configjs = str_replace("\\r","",$configjs);
                        $configjs = preg_replace('/[\n\r]/',' ',$configjs);
                        $configjs = str_replace(" ","",$configjs);
                        $configjs = str_replace("	","",$configjs);

                        $configjs = str_replace(':','":',$configjs);
                        $configjs = str_replace(",",',"',$configjs);
                        $configjs = str_replace("{",'{"',$configjs);
                        $retorno = json_decode($configjs,true);
                    }
                }
            }
        } catch(\Exception $e){ /*var_dump($e->getMessage());*/ }
        return $retorno;
    }

	public function MaiorVersaoDisponivel(Request\Filter $request, Response $response){
        $retorno = Array();
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $params = $this->getParams($params);
            $configIp = json_decode($params['configIp'], true);
            $projectConfig =  json_decode($params['projectConfig'], true);

            $configjs = self::getConfigJs();
            if(!empty($configjs)){
                $rootPath = realpath(str_replace('\\', DIRECTORY_SEPARATOR, __DIR__. '/../../../../../')).DIRECTORY_SEPARATOR;
                $pastasBackend = new \DirectoryIterator($rootPath.'odhenPOS');
                foreach($pastasBackend as $pastaBackend){
                    try {
                        if($pastaBackend->gettype() === 'dir'){
                            $nome = $pastaBackend->getFilename();
                            if(strpos($nome, 'backend_')!==false){
                                if(file_exists($rootPath.'odhenPOS'.DIRECTORY_SEPARATOR.$nome.DIRECTORY_SEPARATOR.'version.json')){
                                    $versionjson = file_get_contents($rootPath.'odhenPOS'.DIRECTORY_SEPARATOR.$nome.DIRECTORY_SEPARATOR.'version.json');
                                    $versionjson = json_decode($versionjson,true);
                                    if(isset($versionjson['backVersion'])){
                                        $versionjson = $versionjson['backVersion'];
                                        //encontra uma versão do backend que seja compativel com o front do projeto
                                        if($versionjson == $configjs['frontVersion']){
                                            $retorno = $configjs;
                                            $retorno['forcaAtualizacao'] = false;
                                            if($projectConfig['frontVersion'] != $configjs['frontVersion'] ){
                                                //Se a versão do front no servidor for difernete da versão do front que esta carregada no navegador  Ex: front carregado no navegar ainda esta naversão 74000. Mas no servidor o front já esta na versão 7500
                                                $retorno['forcaAtualizacao'] = true;
                                            }
                                            if($configIp['serviceUrl'] != $configjs['serviceUrl'] ){
                                                // se o front estiver apontado para uma versão de backend diferente do que esta definido no front do servidor. Ex: front aponta para o back 74000. Mas no servidor o front esta na versão 7500
                                                $retorno['forcaAtualizacao'] = true;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    } catch(\Exception $e){ /*var_dump($e->getMessage());*/ }
                }
            }
        } catch(\Exception $e){ /*var_dump($e->getMessage());*/ }
		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('MaiorVersaoDisponivel', array(array('MaiorVersaoDisponivel' => $retorno))));
	}

    public function logForAPK(Request\Filter $request, Response $response) {
        $params = $request->getFilterCriteria()->getConditions();
        $params = $this->getParams($params);

        $session = $this->getSessionVars(null);

        $this->utilAPI->logForAPK($session['CDFILIAL'], $session['CDCAIXA'], $params['texto']);
    }

    public function logGeraLog(Request\Filter $request, Response $response) {
        $params = $request->getFilterCriteria()->getConditions();
        $params = $this->getParams($params);
        self::logGeraLog_($params);
    }

    public function logGeraLog_($params) {
        try {
			$DATA     = isset($params['DATA'])    ?$params['DATA']    :'';
            $TEXTO    = isset($params['TEXTO'])   ?$params['TEXTO']   :'';
            $NRORG    = isset($params['NRORG'])   ?$params['NRORG']   :'null';
            $CDFILIAL = isset($params['CDFILIAL'])?$params['CDFILIAL']:'null';
            $CDCAIXA  = isset($params['CDCAIXA']) ?$params['CDCAIXA'] :'null';
            $PASTA    = isset($params['PASTA'])   ?$params['PASTA']   :'LogGeraLog';

			$DATA     = !empty($DATA)    ?$DATA    :((new DateTime())->format("d-m-Y H:i:s.v"));
            $NRORG    = !empty($NRORG)   ?$NRORG   :'null';
            $CDFILIAL = !empty($CDFILIAL)?$CDFILIAL:'null';
            $CDCAIXA  = !empty($CDCAIXA) ?$CDCAIXA :'null';
            $CDFILIAL = !empty($CDFILIAL)?$CDFILIAL:'null';
            $PASTA    = !empty($PASTA)   ?$PASTA   :'LogGeraLog';

            if (preg_match('/Linux/', PHP_OS)){
                $DIRETORIO = sys_get_temp_dir();
            } else {
                $DIRETORIO = __DIR__ . DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR;
            }
            if(!realpath($DIRETORIO)){
                throw new \Exception("Erro ao salvar o arquivo de log: ".$DIRETORIO, 1);
            }
            $DIRETORIO = realpath($DIRETORIO);
            $DIRETORIO .= DIRECTORY_SEPARATOR.'LOG';
            if (!file_exists($DIRETORIO)) { mkdir($DIRETORIO, 0777, true); }
            $DIRETORIO .= DIRECTORY_SEPARATOR.$PASTA;
            if (!file_exists($DIRETORIO)) { mkdir($DIRETORIO, 0777, true); }
            if (preg_match('/Linux/', PHP_OS)){
				//shell_exec('find "'.$DIRETORIO.'" -type d -mtime +30 -exec rm -r {} \;');
            } else {
				//shell_exec('forfiles /p "'.$DIRETORIO.'" /d -30 /c "cmd /c if @isdir==TRUE rd /s /q @path"');
            }
            $DIRETORIO .= DIRECTORY_SEPARATOR.'NRORG_'.$NRORG;
            if (!file_exists($DIRETORIO)) { mkdir($DIRETORIO, 0777, true); }
            $DIRETORIO .= DIRECTORY_SEPARATOR.'CDFILIAL_'.$CDFILIAL.'_CDCAIXA_'.$CDCAIXA;
            if (!file_exists($DIRETORIO)) { mkdir($DIRETORIO, 0777, true); }
            $DIRETORIO .= DIRECTORY_SEPARATOR.date("Y_m");
            if (!file_exists($DIRETORIO)) { mkdir($DIRETORIO, 0777, true); }
			$DIRETORIO_temp = realpath($DIRETORIO);
            if(!$DIRETORIO_temp){
                throw new \Exception("Erro ao salvar o arquivo de log: ".$DIRETORIO, 1);
            }
            $DIRETORIO = realpath($DIRETORIO);

            $DIRETORIO = $DIRETORIO.DIRECTORY_SEPARATOR.date("d-m-Y").'.txt';

			$TEXTO = $DATA . " - " . $TEXTO . "\n";

            header('Content-Type: text/plain; charset=utf-8');
            //shell_exec('echo "'.$TEXTO.'" >> "'.$DIRETORIO.'" ');
            if(!file_put_contents($DIRETORIO, $TEXTO, FILE_APPEND)){
                throw new \Exception('Erro ao salvar o arquivo de log: '.$DIRETORIO, 1);
            } else {
                if(isset($response)){
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('LogGeraLog', array(0 => 'Ok')));
                }
            }
        } catch(\Exception $e){
            Exception::logException($e);
            if(isset($response)){
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('LogGeraLog', array(0 => $e->getMessage())));
            } else {
                throw new \Exception($e->getMessage(), 1);
            }
        }
    }

    public function renomearPastaLog($DIRETORIO=null){
        try {
            if (preg_match('/Linux/', PHP_OS)){
                return;
            }
            if(empty($DIRETORIO)){
                $DIRETORIO = __DIR__ . DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR;
                if(!realpath($DIRETORIO)){
                    return;
                }
                $DIRETORIO = realpath($DIRETORIO);
                $DIRETORIO .= DIRECTORY_SEPARATOR.'LOG';
                if(!realpath($DIRETORIO)){
                    if (!file_exists($DIRETORIO)) { mkdir($DIRETORIO, 0777, true); }
                    if(!realpath($DIRETORIO)){
                        return;
                    }
                }
                $DIRETORIO = realpath($DIRETORIO);
                if (!file_exists($DIRETORIO)) { mkdir($DIRETORIO, 0777, true); }
            } else {
                if(!realpath($DIRETORIO)){
                    return;
                }
            }
            if (is_dir($DIRETORIO)) {
                $dataCriacaoPasta = filectime($DIRETORIO);
                $dataAtual = time();
                $diferenca = $dataAtual - $dataCriacaoPasta;
                $dias = $diferenca / (60 * 60 * 24);
                if ($dias >= 90) {
                    $ultimoNome = basename($DIRETORIO);
                    if(empty($ultimoNome)){
                        $ultimoNome='LOG';
                    }
                    $novoDIRETORIO = $ultimoNome.'_backup_'.(date("d_m_Y_H_i_s"));
                    exec('ren "'.$DIRETORIO.'" "'.$novoDIRETORIO.'" ');
                }
            }
        } catch(\Exception $e){
            Exception::logException($e);
        }
    }
}
